import { authenticate } from "../shopify.server";
import { useFetcher, useLoaderData } from '@remix-run/react';
import { reviewListingData, defaultReviewData } from "./service";
import { XIcon } from '@shopify/polaris-icons';
import { useState, useEffect } from "react";
import { Icon, SkeletonThumbnail } from "@shopify/polaris";
import clearFilterIcon from '../assets/images/icon-clear-filter.png';
import Swal from 'sweetalert2';
function extractID(id) {
    const extractedId = id.split('/');
    return extractedId[extractedId.length - 1];
}

function formatDate(dateTimeString) {
    const date = new Date(dateTimeString);
    return date.toISOString().split('T')[0];
}

export const loader = async ({ request }) => {
    const { admin } = await authenticate.admin(request);
    const getStoreId = async () => {
        const response = await admin.graphql(
            `#graphql
              query { shop { id } }`,
        );
        const responseJson = await response.json();
        const storeID = responseJson.data.shop.id;
        return extractID(storeID);
    }
    const fetchAllProducts = async (cursor = null) => {
        const query = `
            query GetProducts($first: Int!, $after: String) {
            products(first: $first, after: $after, query: "status:active") {
                edges {
                node {
                    id
                    title
                }
                }
                pageInfo {
                hasNextPage
                endCursor
                }
            }
            }
        `;
        const variables = { first: 250, after: cursor };
        const response = await admin.graphql(query, { variables });
        const responseJson = await response.json();

        const products = responseJson.data.products.edges.map(edge => edge.node);
        const hasNextPage = responseJson.data.products.pageInfo.hasNextPage;
        const endCursor = responseJson.data.products.pageInfo.endCursor;

        if (hasNextPage) {
            const moreProducts = await fetchAllProducts(endCursor);
            return [...products, ...moreProducts];
        } else {
            return products;
        }
    };
    const fetchAllCustomers = async (cursor = null) => {
        const query = `
            query GetCustomers($first: Int!, $after: String) {
                customers(first: $first, after: $after) {
                    edges {
                        node {
                            id
                            firstName
                            lastName
                        }
                    }
                    pageInfo {
                        hasNextPage
                        endCursor
                    }
                }
            }
        `;
        const variables = { first: 250, after: cursor };
        const response = await admin.graphql(query, { variables });
        const responseJson = await response.json();

        const customers = responseJson.data.customers.edges.map(edge => edge.node);
        const hasNextPage = responseJson.data.customers.pageInfo.hasNextPage;
        const endCursor = responseJson.data.customers.pageInfo.endCursor;

        if (hasNextPage) {
            const moreCustomers = await fetchAllCustomers(endCursor);
            return [...customers, ...moreCustomers];
        } else {
            return customers;
        }
    };

    const allCustomers = await fetchAllCustomers();
    const allProducts = await fetchAllProducts();
    const storeID = await getStoreId();
    return ({ products: allProducts, customers: allCustomers, storeID: storeID });
};

export const action = async ({ request }) => {
    const { admin } = await authenticate.admin(request);
    const formData = new URLSearchParams(await request.text());
    const productIds = formData.getAll('productId');
    const productsId = productIds[0].split(',');
    const updatedProductIds = [];
    for (let index = 0; index < productsId.length; index++) {
        const element = "gid://shopify/Product/" + productsId[index];
        updatedProductIds.push(element);
    }
    const query = `
      query GetProductTitles($ids: [ID!]!) {
        nodes(ids: $ids) {
          ... on Product {
            title
          }
        }
      }
    `;
    const response = await admin.graphql(query, {
        variables: { ids: updatedProductIds },
    });
    const responseJson = await response.json();
    return (responseJson.data.nodes);
};

function Pagination({ totalPage, currentPage, handlePageClick, handleNextPage, handlePreviousPage }) {
    let pageNumberList = [];
    if (currentPage == reviewListingData.initialPageNumber && totalPage == reviewListingData.paginationMinimumCount || currentPage == totalPage && totalPage == reviewListingData.paginationMinimumCount) {
        pageNumberList = reviewListingData.pageNumberListForTwoPage;
    } else {
        let startIndex = 0, endIndex = 0;
        if (currentPage == reviewListingData.initialPageNumber && totalPage >= reviewListingData.numberOfPageToShow ) {
            startIndex = currentPage;
            endIndex = reviewListingData.numberOfPageToShow;
        } else if (currentPage == totalPage ) {
            startIndex = currentPage - (reviewListingData.numberOfPageToShow - 1);
            endIndex = currentPage;
        } else {
            startIndex = currentPage - 1;
            endIndex = currentPage + 1;
        }
        for (let index = startIndex; index <= endIndex; index++) {
            pageNumberList.push(index);
        }
    }
    return (
        <>
            <div className="vins-pagination-container">
                <ul className="vins-unstylished-list vins-pagination-list">
                    <li className={currentPage == 1 ? "vins-pagination-icon vins-inactive-page-navigation" : "vins-pagination-icon"} onClick={currentPage !== 1 ? handlePreviousPage : undefined }>{"<"}</li>
                    {pageNumberList.map((value) => (
                        <li key={value} className={value == currentPage ? "vins-active-page" : ""} value={value} onClick={() => handlePageClick(value)}>{value}</li>
                    ))}
                    <li className={currentPage == totalPage ? "vins-pagination-icon vins-inactive-page-navigation" : "vins-pagination-icon"} onClick={currentPage !== totalPage ? handleNextPage : undefined}>{">"}</li>
                </ul>
            </div>
        </>
    )
}

export default function ReviewListing() {
    const loaderData = useLoaderData();
    const allProducts = loaderData.products;
    const allCustomers = loaderData.customers;
    const storeID = loaderData.storeID;
    const [reviewData, setReviewData] = useState([]);
    const [isLoading, setIsLoading] = useState(defaultReviewData.isLoadingValue);
    const [totalPage, setTotalPage] = useState(reviewListingData.totalPage);
    const fetcher = useFetcher();
    const limit = reviewListingData.pageDataLimit;
    const [page, setPage] = useState(reviewListingData.initialPageNumber);
    const [reviewProductTitle, setReviewProductTitle] = useState([]);
    const [filterValues, setFilterValues] = useState({
        "vins-product-list-dropdown": '',
        "vins-customer-list-dropdown": '',
        "vins-star-rating-list-dropdown": '',
        "vins-sort-by-list-dropdown": 'Newest',
    })

    const fetchProductDetails = (productID, totalReviewCount) => {
        if (totalReviewCount == 0) {
            setIsLoading(false);
        } else {
            fetcher.submit(
                {
                    productId: productID,
                },
                {
                    method: 'post',
                }
            );
        }
    };
    const handleFilterChange = (event) => {
        const { name, value } = event.target;
        setFilterValues((prevState) => ({
            ...prevState,
            [name]: value
        }));
        setIsLoading(true);
    };
    const handleClearAll = () => {
        setFilterValues({
            "vins-product-list-dropdown": '',
            "vins-customer-list-dropdown": '',
            "vins-star-rating-list-dropdown": '',
            "vins-sort-by-list-dropdown": 'Newest',
        })
        setIsLoading(true);
    }
    const handlePageClick = (value) => {
        setPage(value);
        setIsLoading(defaultReviewData.isLoadingValue);
    }
    const handlePreviousPage = () => {
        setPage(page - 1);
        setIsLoading(defaultReviewData.isLoadingValue);
    }
    const handleNextPage = () => {
        setPage(page + 1);
        setIsLoading(defaultReviewData.isLoadingValue);
    }

    const skeletonRows = Array.from({ length: limit }, (_, rowIndex) => (
        <tr className="vins-review-list-skeleton-row" key={rowIndex}>
            {Array.from({ length: reviewListingData.tableColumnSize }, (_, colIndex) => (
                <td key={colIndex}>
                    <SkeletonThumbnail />
                </td>
            ))}
        </tr>
    ));
    const handleInvalidateReview = (event) => {
        const button = event.currentTarget;
        Swal.fire({
            title: "Are you sure?",
            text: "You won't be able to revert this review!",
            icon: "warning",
            showCancelButton: true,
            confirmButtonColor: "#3085d6",
            cancelButtonColor: "#d33",
            confirmButtonText: "Yes",
            cancelButtonText: "No"
        }).then((result) => {
            if (result.isConfirmed) {
                const myHeaders = new Headers();
                myHeaders.append("Content-Type", "application/x-www-form-urlencoded");

                const urlencoded = new URLSearchParams();
                urlencoded.append("storeId", storeID);
                urlencoded.append("call", "postInvalidateReview");
                urlencoded.append("productId", button.dataset.productId);
                urlencoded.append("userId", button.dataset.userId);
                const requestOptions = {
                    method: "POST",
                    headers: myHeaders,
                    body: urlencoded,
                    redirect: "follow"
                };

                fetch("https://shopifydev.vinsinfo.in/ShopifyReviewApp/admin_review_functions.php", requestOptions)
                    .then((response) => response.json())
                    .then((result) => {
                        if (result.status_code == 200) {
                            Swal.fire({
                                title: "Removed!",
                                text: "Review invalidated successfully.",
                                icon: "success",
                                showConfirmButton: false
                            });
                            handleClearAll();
                        }
                    })
                    .catch((error) => console.error(error));

            }
        });
    }
    const getReview = () => {
        const myHeaders = new Headers();
        myHeaders.append("Content-Type", "application/x-www-form-urlencoded");

        const urlencoded = new URLSearchParams();
        urlencoded.append("storeId", storeID);
        urlencoded.append("call", "get_review");
        urlencoded.append("limit", limit);
        urlencoded.append("page", page);
        urlencoded.append("productId", filterValues["vins-product-list-dropdown"]);
        urlencoded.append("userId", filterValues["vins-customer-list-dropdown"]);
        if (filterValues["vins-star-rating-list-dropdown"] >= 1 && filterValues["vins-star-rating-list-dropdown"] <= 5) {
            urlencoded.append("filtersByStarCount", filterValues["vins-star-rating-list-dropdown"]);
        }
        if (filterValues["vins-sort-by-list-dropdown"] === reviewListingData.sortByNewest) {
            urlencoded.append("dateSort", "1");
            urlencoded.append("dateSortOrder", "0");
        }
        if (filterValues["vins-sort-by-list-dropdown"] === reviewListingData.sortByOldest) {
            urlencoded.append("dateSort", "1");
            urlencoded.append("dateSortOrder", "1");
        }
        if (filterValues["vins-sort-by-list-dropdown"] === reviewListingData.sortByRatingHighest) {
            urlencoded.append("star_sort", "1");
            urlencoded.append("sort_by_star_asc", "0");
        }
        if (filterValues["vins-sort-by-list-dropdown"] === reviewListingData.sortByRatingOldest) {
            urlencoded.append("star_sort", "1");
            urlencoded.append("sort_by_star_asc", "1");
        }

        const requestOptions = {
            method: "POST",
            headers: myHeaders,
            body: urlencoded,
            redirect: "follow"
        };

        fetch("https://shopifydev.vinsinfo.in/ShopifyReviewApp/admin_review_functions.php", requestOptions)
            .then((response) => response.json())
            .then((result) => {
                const reviewDataUpdate = [];
                const productIds = []
                for (let index = 0; index < result.reviews.length; index++) {
                    productIds.push(result.reviews[index].productId);
                    const reviewData = { "customerName": result.reviews[index].name, "rating": result.reviews[index].starCount, "reviewHeading": result.reviews[index].reviewTitle, "reviewContent": result.reviews[index].reviewText, "recommended": result.reviews[index].recommendProduct == 1 ? "Yes" : "No", "reviewedDate": formatDate(result.reviews[index].date), "productID": result.reviews[index].productId, "userID": result.reviews[index].userId };
                    reviewDataUpdate.push(reviewData);
                }
                fetchProductDetails(productIds, result.total_reviews);
                setReviewData(reviewDataUpdate);
                setTotalPage(Math.ceil(result.total_reviews / limit));
            })
            .catch((error) => console.error(error));
    }
    useEffect(() => {
        getReview();
    }, [filterValues, page])

    useEffect(() => {
        if (fetcher.data) {
            setReviewProductTitle(fetcher.data);
            setIsLoading(false);
        }
    }, [fetcher.data])

    return (
        <>
            <div className="vins-review-listing">
                <div className="vins-review-listing-headign-container">
                    <select name="vins-product-list-dropdown" id="vins-product-list-dropdown" onChange={handleFilterChange}>
                        <option value="">Product Name</option>
                        {!isLoading &&
                            <>
                                {allProducts.map((allProduct) => (
                                    <option key={extractID(allProduct.id)} value={extractID(allProduct.id)} selected={filterValues["vins-product-list-dropdown"] === extractID(allProduct.id)}>
                                        {allProduct.title}
                                    </option>
                                ))}
                            </>
                        }
                    </select>
                    <select name="vins-customer-list-dropdown" id="vins-customer-list-dropdown" onChange={handleFilterChange}>
                        <option value="">Customer Name</option>
                        {!isLoading &&
                            <>
                                {allCustomers.map((allCustomer) => (
                                    <option key={extractID(allCustomer.id)} value={extractID(allCustomer.id)} selected={filterValues["vins-customer-list-dropdown"] === extractID(allCustomer.id)}>
                                        {allCustomer.firstName} {allCustomer.lastName}
                                    </option>
                                ))}
                            </>
                        }
                    </select>
                    <select name="vins-star-rating-list-dropdown" id="vins-star-rating-list-dropdown" onChange={handleFilterChange}>
                        <option >All Ratings</option>
                        {reviewListingData.sortByStarRatingValue.map((rating) => (
                            <option key={rating} value={rating} selected={filterValues["vins-star-rating-list-dropdown"] === rating}>
                                {rating}
                            </option>
                        ))}
                    </select>
                    <select name="vins-sort-by-list-dropdown" id="vins-sort-by-list-dropdown" onChange={handleFilterChange}>
                        {Object.entries(reviewListingData.sortByValues).map(([value, label]) => (
                            <option key={value} value={value} selected={filterValues["vins-sort-by-list-dropdown"] === value}>{label}</option>
                        ))}
                    </select>

                    <button onClick={handleClearAll}>Clear All</button>
                </div>
                <div className="vins-review-listing-content-container">
                    <table className="vins-review-listing-table">
                        <thead>
                            <tr>
                                <th className="vins-review-list-product">Product Name</th>
                                <th className="vins-review-list-customer">Customer Name</th>
                                <th className="vins-review-list-star-rating">Rating</th>
                                <th className="vins-review-list-star-heading">Rating Heading</th>
                                <th className="vins-review-list-content">Review Content</th>
                                <th className="vins-review-list-recommended">Recommended</th>
                                <th className="vins-review-list-date">Review Date</th>
                                <th className="vins-review-invalidate">Reject Reviews</th>
                            </tr>
                        </thead>
                        <tbody>
                            {isLoading ? (
                                skeletonRows
                            ) : (
                                <>
                                    {reviewData.map((review, index) => (
                                        <tr key={index}>
                                            <td>{reviewProductTitle[index].title}</td>
                                            <td>{review.customerName}</td>
                                            <td className="vins-start-rating vins-center" style={{ '--rating': review.rating }} >☆☆☆☆☆</td>
                                            <td>{review.reviewHeading}</td>
                                            <td>{review.reviewContent}</td>
                                            <td className="vins-center">{review.recommended}</td>
                                            <td>{review.reviewedDate}</td>
                                            <td><button className="vins-review-invalide-button" data-product-id={review.productID} data-user-id={review.userID} onClick={handleInvalidateReview}><Icon source={XIcon} /></button></td>
                                        </tr>
                                    ))}
                                </>
                            )}
                        </tbody>
                    </table>
                    <div className="vins-pagination">
                        {isLoading && totalPage >= reviewListingData.paginationMinimumCount &&
                            <SkeletonThumbnail className="vins-pagination-skeleton" />
                        }
                        {!isLoading && totalPage >= reviewListingData.paginationMinimumCount &&
                            <Pagination totalPage={totalPage} currentPage={page} handlePageClick={handlePageClick} handleNextPage={handleNextPage} handlePreviousPage={handlePreviousPage} />
                        }
                    </div>
                </div>
            </div>
        </>
    )
}

